<?php

/*

    ユーティリティ関数

*/


// crontab 設定時の例外
// エラーを出した予約設定のIDとメッセージを収容する
class ExceptionCrontab extends exception {

    private $errs;

    public function set( $id, $msg ){

        $this->errs[] = array( $id, $msg );
    }

    public function get_errs(){ return $this->errs; }

    public function count(){ return count( $this->errs ); }
}


// ロック用クラス
class Lock
{
    private $dir;

    public function __construct( $dir ){

        $this->dir = $dir;

        $lock = get_vardir().$this->dir;
        $timeout = LOCKTIMEOUT;
        while( ! @mkdir( $lock ) ){

            if( ! $timeout ){

                if( ! @rmdir( $lock ) ) throw new Exception( 'ロックディレクトリの削除に失敗しました。パーミッションを確認して下さい。' );
                break;
            }
            else{
                sleep( 1 );
                --$timeout;
            }
        }

        show_debug( 'lock: '.$lock );
    }


    function __destruct()
    {
        $lock = get_vardir().$this->dir;
        if( file_exists( $lock ) ) rmdir( $lock );

        show_debug( 'unlock: '.$lock );
    }

}


// デフォルトステータス値
function get_default_status()
{
    return array( 
                  0, // ID
                  0, // PID
                  0, // CHANNEL
                  0, // STARTTIME
                  0, // DURATION
                  '', // OUTDIR
                  '', // NAMEBASE
                  0, // UDP
                  '', // IP
                  0, // RESERV
                  0 // PORT
        );
}


// デフォルト録画設定値
function get_default_setting()
{
    return array(
                  0, // SET_ID
                  '', // SET_COMMENT
                  0, // SET_CHANNEL

                  0, // SET_YEAR
                  0, // SET_MONTH
                  0, // SET_DAY
                  0, // SET_HOUR
                  0, // SET_MIN

                  0, // SET_SUN
                  0, // SET_MON
                  0, // SET_TUE
                  0, // SET_WED
                  0, // SET_THU
                  0, // SET_FRI
                  0, // SET_SAT

                  3600, // SET_DURATION
                  SPOOL_DIRECTORY, // SET_OUTDIR
                  '', // SET_NAMEBASE
                  0, // SET_OVERWRITE
                  0, // SET_UDP
                  '', // SET_IP
                  1234, // SET_PORT

                  1, // SET_ACTIVE
                  MODE_ONCE, // SET_MODE
                  0, // SET_SLEEPSEC
                  0, // SET_SUSPEND

                  0, // SET_RES1
                  0, // SET_RES2
                  0  // SET_RES3
        );
}


// 予約のチューナへの割り当て情報のデフォルト値
function get_default_tuneritem()
{
    return array( 
                  0, // TN_ID
                  0, // TN_START_W
                  0  // DURATION
        );
}


// ホームディレクトリ取得
function get_homedir()
{
    $home = HOME;
    if( $home[ strlen( $home ) -1 ] != '/' ) $home = $home.'/';
    return $home;
}


// 作業ディレクトリパス取得
function get_vardir()
{
    return get_homedir().VARDIR.'/';
}


// キャッシュディレクトリパス取得
function get_cachedir()
{
    return get_homedir().CACHEDIR.'/';
}


// 設定ファイル名取得
function get_settingfile()
{
    return get_vardir().SETTINGFILE;
}


// 予約ファイル名取得
function get_reservfile()
{
    return get_vardir().RESERVFILE;
}


// ステータスファイル名取得
function get_statusfile()
{
    return get_vardir().STATUSFILE;
}


// crontabファイル名取得
function get_crontabfile()
{
    return get_vardir().CRONTABFILE;
}


// ログファイル名取得
function get_logfile()
{
    return get_vardir().LOGFILE;
}


// サスペンド状態ファイル名取得
function get_suspendfile()
{
    return get_vardir().SUSPENDFILE;
}


// ID から録画情報を取得
// 無ければ FALSE を返す
function get_setting_by_id( $id, &$setting_array )
{
    show_debug( 'get_setting_by_id id ='.$id );

    foreach( $setting_array as $setting ){

        if( $setting[ SET_ID ] == $id ){
            show_debug( 'found' );
            return $setting;
        }
    }

    return FALSE;
}


// POST 時に録画設定項目を更新する
function update_settingitem( &$setting )
{
    global $CHANNEL_TABLE;

    $id = $setting[ SET_ID ];
    show_debug( 'update_settingitem id = '.$id );

    $setting[ SET_ERRCODE ] = 0;

    // アクティブ
    if( isset( $_POST[ 'active'.$id ] ) ) $setting[ SET_ACTIVE ] = 0;
    else $setting[ SET_ACTIVE ] = 1;

    // コメント
    if( isset( $_POST[ 'comment'.$id ] ) ) $setting[ SET_COMMENT ] = trim( $_POST[ 'comment'.$id ] );

    // チャンネル
    if( isset( $_POST[ 'channel'.$id ] ) ){

        $channel = -1;
        foreach( $CHANNEL_TABLE as $ch => $chname ){

            if( $_POST[ 'channel'.$id ] == $chname.'('.$ch.')' ){
                $channel = $ch;
                break;
            }
        }

        $setting[ SET_CHANNEL ] = $channel;
    }

    // 録画のモード
    if( isset( $_POST[ 'type'.$id ] ) ){
        if( $_POST[ 'type'.$id ] == 'once' ) $setting[ SET_MODE ] = MODE_ONCE;
        else $setting[ SET_MODE ] = MODE_WEEK;
    }

    // 日時
    if( $setting[ SET_MODE ] == MODE_ONCE ){
        if( isset( $_POST[ 'month'.$id ] ) ) $setting[ SET_MONTH ] = $_POST[ 'month'.$id ];
        if( isset( $_POST[ 'day'.$id ] ) ) $setting[ SET_DAY ] = $_POST[ 'day'.$id ];
    }

    // 曜日
    if( $setting[ SET_MODE ] == MODE_WEEK ){
        if( isset( $_POST[ 'sun'.$id ] ) ) $setting[ SET_SUN ] = 1; else $setting[ SET_SUN ] = 0;
        if( isset( $_POST[ 'mon'.$id ] ) ) $setting[ SET_MON ] = 1; else $setting[ SET_MON ] = 0;
        if( isset( $_POST[ 'tue'.$id ] ) ) $setting[ SET_TUE ] = 1; else $setting[ SET_TUE ] = 0;
        if( isset( $_POST[ 'wed'.$id ] ) ) $setting[ SET_WED ] = 1; else $setting[ SET_WED ] = 0;
        if( isset( $_POST[ 'thu'.$id ] ) ) $setting[ SET_THU ] = 1; else $setting[ SET_THU ] = 0;
        if( isset( $_POST[ 'fri'.$id ] ) ) $setting[ SET_FRI ] = 1; else $setting[ SET_FRI ] = 0;
        if( isset( $_POST[ 'sat'.$id ] ) ) $setting[ SET_SAT ] = 1; else $setting[ SET_SAT ] = 0;
    }

    // 時刻
    if( isset( $_POST[ 'hour'.$id ] ) ) $setting[ SET_HOUR ] = (int) $_POST[ 'hour'.$id ];    
    if( isset( $_POST[ 'min'.$id ] ) ) $setting[ SET_MIN ] = (int) $_POST[ 'min'.$id ];    

    // 録画時間
    if( isset( $_POST[ 'duration'.$id ] ) ) $setting[ SET_DURATION ] = (int)( $_POST[ 'duration'.$id ] )* 60;

    // スリープ時間
    if( isset( $_POST[ 'sleepsec'.$id ] ) ) $setting[ SET_SLEEPSEC ] = 60 - (int)$_POST[ 'sleepsec'.$id ];

    // ディレクトリ
    $setting[ SET_OUTDIR ] = trim( SPOOL_DIRECTORY );

    // ファイル名
    if( isset( $_POST[ 'namebase'.$id ] ) ) $setting[ SET_NAMEBASE ] = trim( $_POST[ 'namebase'.$id ] );
    if( isset( $_POST[ 'overwrite'.$id ] ) ) $setting[ SET_OVERWRITE ] = 0;
    else $setting[ SET_OVERWRITE ] = 1;

    // UDP送信
    if( isset( $_POST[ 'udp'.$id ] ) ) $setting[ SET_UDP ] = 1;
    else $setting[ SET_UDP ] = 0;
    if( isset( $_POST[ 'ip'.$id ] ) ) $setting[ SET_IP ] = trim( $_POST[ 'ip'.$id ] );
    if( isset( $_POST[ 'port'.$id ] ) ) $setting[ SET_PORT ] = (int) $_POST[ 'port'.$id ];

    // サスペンド
    if( isset( $_POST[ 'suspend'.$id ] ) ) $setting[ SET_SUSPEND ] = 1;
    else $setting[ SET_SUSPEND ] = 0;
}

    
// 録画設定を更新する
function update_setting( &$setting_array, $operate, $maxnumber )
{
    $lock = new Lock( FILELOCKDIR );

    show_debug( 'update_setting' );

    $updated = FALSE;
    for( $settingno = 0; $settingno < $maxnumber; ++$settingno ){

        // 未設定の時はデフォルト値をセット
        if( count( $setting_array ) <= $settingno ){

            $setting_array[ $settingno ] = get_default_setting();
            $setting_array[ $settingno ][ SET_ID ] = time()+$settingno;
            $updated = TRUE;
        }

        $setting = &$setting_array[ $settingno ];

        // 設定が壊れていたらデフォルト値をセット
        if( ! is_array( $setting ) || count( $setting ) != SET_SIZE ){

            show_debug( 'no.'.$settingno.' is broken' );

            $setting = get_default_setting();
            $setting[ SET_ID ] = time()+$settingno;
            $updated = TRUE;
        }

        // POST された値を入力
        if( $operate == OP_REC || $operate == OP_PLAY || $operate == OP_SAVE ){

            update_settingitem( $setting );
            $updated = TRUE;
        }
    }

    if( $updated ) save_csv( get_settingfile(), $setting_array );
}


// POST時に予約設定を更新する
// 戻り値は更新した予約設定
function &update_reserv( &$setting_array, $id )
{
    $lock = new Lock( FILELOCKDIR );

    show_debug( 'update_reserv id = '.$id );

    $new = FALSE;
    $setting = FALSE;
    for( $settingno = 0; $settingno < count( $setting_array ); ++$settingno ){

        if( $setting_array[ $settingno ][ SET_ID ] == $id ){
            $setting = &$setting_array[ $settingno ];
            break;
        }
    }

    // 設定が見付からなければ新規に追加
    if( ! $setting ){

        show_debug( 'append new setting' );

        $setting_array[] = get_default_setting();
        $setting = &$setting_array[ count( $setting_array ) -1 ];
        $setting[ SET_ID ] = $id;
    }

    update_settingitem( $setting );

    return $setting;
}


// 予約設定を削除する
function remove_reserv( &$setting_array, $id )
{
    $lock = new Lock( FILELOCKDIR );

    $delno = FALSE;
    for( $settingno = 0; $settingno < count( $setting_array ); ++$settingno ){

        if( $setting_array[ $settingno ][ SET_ID ] == $id ){
            $delno = (int)$settingno+1;
            break;
        }
    }
    show_debug( 'remove_reserv id = '.$id.' delno = '.($delno-1) );

    if( $delno ) array_splice( $setting_array, $delno-1, 1 );
}


// POST 時に操作を取得
// 戻り値 操作名
// $id に ID が入る
function get_operate( &$id, $maxnumber )
{
    $operate = OP_NONE;
    $id = FALSE;

    if( $_SERVER[ "REQUEST_METHOD" ] == 'POST' ){

        $settingno = '';

        if( isset( $_POST[ 'save' ] ) ) $operate = OP_SAVE;

        else if( isset( $_POST[ 'update' ] ) ) $operate = OP_UPDATE;

        else if( isset( $_POST[ 'do_del'  ] ) ) $operate = OP_DO_DEL;

        else if( isset( $_POST[ 'stop_spd'  ] ) ) $operate = OP_STOP_SPD;

        else{
            
            for( $i = 0; $i < $maxnumber ; ++$i ){

                if( isset( $_POST[ 'play'.$i ] ) ){
                    $operate = OP_PLAY;
                    $settingno = $i;
                    break;
                }
                else if( isset( $_POST[ 'rec'.$i  ] ) ){
                    $operate = OP_REC;
                    $settingno = $i;
                    break;
                }

                // 停止の場合のIDは PID を表す
                else if( isset( $_POST[ 'stop'.$i  ] ) ){
                    $operate = OP_STOP;
                    $settingno = $i;
                    break;
                }
                else if( isset( $_POST[ 'edit'.$i  ] ) ){
                    $operate = OP_EDIT;
                    $settingno = $i;
                    break;
                }
                else if( isset( $_POST[ 'del'.$i  ] ) ){
                    $operate = OP_DEL;
                    $settingno = $i;
                    break;
                }
            }
        }

        if( isset( $_POST[ 'id'.$settingno ] ) ) $id = $_POST[ 'id'.$settingno ];
    }

    show_debug( 'get_operate : operate = '.$operate.' id = '.$id );
    return $operate;
}


// csvファイル読み込み
function read_csv( $file, &$data )
{
    show_debug( 'read_csv : '.$file );

    $data = array();

    if( ! file_exists( $file ) ){
        throw new Exception( 'エラー(read_csv) :ファイル('.$file.')が存在しません。' );
    }

    if( ! ( $fp = fopen( $file, 'r') ) ){
        throw new Exception( 'エラー(read_csv) :ファイル('.$file.')のオープンに失敗しました。パーミッションを確認してください。' );
    }
    while( $tmp = fgetcsv( $fp ) ) $data[] = $tmp;
    fclose( $fp );

    return TRUE;
}


// csvファイル書き込み
function save_csv( $file, &$data )
{
    show_debug( 'save_csv : '.$file );

    if( ! is_array( $data ) ) return FALSE;

    if( ! ( $fp = fopen( $file, 'w+') ) ){
        throw new Exception( 'エラー(save_csv) :ファイル('.$file.')のオープンに失敗しました。パーミッションを確認してください。' );
    }

    foreach( $data as $line ) fputcsv( $fp, $line );
    fclose( $fp );

    return TRUE;
}


// プロセスが動いているか
function is_running( $pid )
{
    if( $pid ){

        show_debug( 'is_running : pid = '.$pid );

        $ret = exec( 'ps --no-headers -p '.$pid );
        show_debug( $ret );

        if( $ret ) return TRUE;
    }

    return FALSE;
}


// ログ保存
function write_log( $log )
{
    $log = date( 'y/m/d H:i:s' ,time() ).' : '.$log."\n";

    if( ( $fp = fopen( get_logfile(), 'a' ) ) ) {
        fputs( $fp, $log  );
        fclose( $fp );
    }
    else throw new Exception( 'エラー(write_log): ログファイルのオープンに失敗しました。' );
}


// 予約設定から次の録画日時が正しいか調べる
function check_date( &$setting )
{
    $time = time();
    $year = (int)date( 'Y', $time );
    if( mktime( $setting[ SET_HOUR ], $setting[ SET_MIN ], 0, $setting[ SET_MONTH ], $setting[ SET_DAY ], $year ) < $time ) ++$year;

    show_debug( 'check_date : '.$year.'/'.$setting[ SET_MONTH ].'/'.$setting[ SET_DAY ] );

    if( $setting[ SET_MONTH ] <= 0 || $setting[ SET_MONTH ] > 12  || $setting[ SET_DAY ] <= 0 
        || ! checkdate( $setting[ SET_MONTH ], $setting[ SET_DAY ], $year ) ) return FALSE;

    return TRUE;
}


// 予約設定から次の録画日時を求める
// 戻り値は UNIX 時刻
function get_nexttime( &$setting, $time )
{
    $year = (int)date( 'Y', $time );

    // 日付指定
    if( $setting[ SET_MODE ] == MODE_ONCE ){

           $nexttime = mktime( $setting[ SET_HOUR ], $setting[ SET_MIN ], 0, $setting[ SET_MONTH ], $setting[ SET_DAY ], $year );

           // 来年か ?
           if( $nexttime < $time ) $nexttime = mktime( $setting[ SET_HOUR ], $setting[ SET_MIN ], 0, $setting[ SET_MONTH ], $setting[ SET_DAY ], $year+1 );
    }

    // 曜日指定
    else{

        $month = (int)date( 'm', $time );
        $day = (int)date( 'd', $time );
        $hour = (int)date( 'H', $time );
        $min = (int)date( 'i', $time );
        $week = (int)date( 'w', $time );

        $nexttime = mktime( $setting[ SET_HOUR ], $setting[ SET_MIN ], 0, $month, $day, $year );

        // 今日
        if( $setting[ SET_SUN + $week ] 
            && ( $setting[ SET_HOUR ] > $hour || ( $setting[ SET_HOUR ] == $hour && $setting[ SET_MIN ] > $min ) ) ){}

        // 明日以降
        else{

            for( $i = 1; $i < 8; ++$i ){
    
                $nexttime += ( 3600 * 24 );

                if( $setting[ SET_SUN + ( $week + $i ) % 7 ] ) break;
            }
        }
    }

    return $nexttime;
}


// 予約設定の配列から次の録画日時を求める
// 戻り値は UNIX 時刻
function get_nexttime_array( &$setting_array )
{
    $time = time();
    $nexttime = 0;

    foreach( $setting_array as $setting ){

        if( $setting[ SET_ACTIVE ] ){

            $nexttime_tmp = get_nexttime( $setting, $time );
            if( ! $nexttime || $nexttime_tmp < $nexttime ) $nexttime = $nexttime_tmp;
        }
    }

    show_debug( 'get_nexttime_array = '.date( 'Y/m/d H:i:D', $nexttime ) );

    return $nexttime;
}


// エラーメッセージ
function get_errmsg( &$setting, &$reserv_array )
{
    $errcode = $setting[ SET_ERRCODE ];

    $errmsg = 'エラー : <br />';
    if( $errcode & ERR_MODE ) $errmsg = $errmsg.'録画モードが指定されていません。<br />';
    if( $errcode & ERR_DATE ) $errmsg = $errmsg.'日付が間違っています。<br />';
    if( $errcode & ERR_WEEK ) $errmsg = $errmsg.'曜日が指定されていません。<br />';
    if( $errcode & ERR_TIME ) $errmsg = $errmsg.'時刻の範囲が間違っています。<br />';
    if( $errcode & ERR_DURATION ) $errmsg = $errmsg.'録画時間が正しくありません。<br />';
    if( $errcode & ERR_SLEEP ) $errmsg = $errmsg.'マージン時間が正しくありません。<br />';
    if( $errcode & ERR_DIR ) $errmsg = $errmsg.'ディレクトリが指定されていません。<br />';
    if( $errcode & ERR_NAME ) $errmsg = $errmsg.'ファイル名が指定されていません。<br />';
    if( $errcode & ERR_IP ) $errmsg = $errmsg.'IPが指定されていません。<br />';
    if( $errcode & ERR_PORT ) $errmsg = $errmsg.'ポートが指定されていません。<br />';
    if( $errcode & ERR_DUP ) $errmsg = $errmsg.'録画時刻が重複しています。<br />';
    if( $errcode & ERR_BROKEN ) $errmsg = $errmsg.'録画設定ファイルが壊れています。<br />';

     // 重複表示
     if( is_array( $reserv_array ) && ( $setting[ SET_ERRCODE ] & ERR_DUP  ) ){

         $week_array = array( '日', '月', '火', '水', '木', '金', '土' );
         $errmsg = $errmsg.'<br />重複 : <br />';

         foreach( $reserv_array as $setting_tmp ){

             if( $setting[ SET_ID ] != $setting_tmp[ SET_ID ] && ( $setting_tmp[ SET_ERRCODE ] & ERR_DUP ) ){

                 $name = $setting_tmp[ SET_COMMENT ];
                 if( ! $name ) $name = $setting_tmp[ SET_NAMEBASE ];


                 $date = '';
                 if( $setting_tmp[ SET_MODE ] == MODE_WEEK ){

                     $week = '';
                     for( $i = 0; $i < 7; ++$i ) if( $setting_tmp[ SET_SUN + $i ] ) $week = $week.$week_array[ $i ];
                     $date = '週間('.$week.')';
                 }
                 else if( $setting_tmp[ SET_MODE ] == MODE_ONCE ){

                     $nexttime = get_nexttime( $setting_tmp, time() );
                     $date = date( 'y/n/d', $nexttime ).'('.$week_array[ (int)date( 'w', $nexttime ) ].')';
                 }

                 $time = $setting_tmp[ SET_HOUR ].'時'.$setting_tmp[ SET_MIN ].'分';
                 $duration = ( (int)$setting_tmp[ SET_DURATION ]/60 ).' 分';

                 $errmsg = $errmsg.$name.' | '.$date.' | 開始時刻'.$time.' | 録画時間 '.$duration.'<br />';
             }
         }
     }


    return $errmsg;
}


// サスペンドスクリプト起動
function exec_suspend()
{
    if( ! ENABLESUSPEND ) return FALSE;

    $suspendfile = get_suspendfile();

    if( file_exists( $suspendfile ) ){

        $status = file_get_contents( $suspendfile );
        $tmp = explode( ',', $status );
        $time = (int)$tmp[ 0 ];
        $pid = (int)$tmp[ 1 ];

        // スクリプトが起動中なのでキャンセル
        if( time() - $time <= ( SUSPENDSEC + 5 ) ) return FALSE;

        // 通常は do-suspend.sh の中で $suspendfile を削除するが、何らかの理由で残ってた場合はここで削除
        else unlink( $suspendfile );
    }


    //////////////////////


    // ローカルホスト
    $localhost = 'http://localhost';
    if( isset( $_SERVER['SERVER_PORT'] ) ) $localhost = $localhost.':'.$_SERVER['SERVER_PORT'];

    // do-suspend.phpまでのパス
    $serverpath = $_SERVER['PHP_SELF'];
    $serverpath = dirname( $serverpath ).'/do-suspend.php';

    // 実行スクリプトの前に ./ が付いていたら取り除いてフルパスにする
    $script = get_homedir().str_replace( './', '', SUSPEND_SCRIPT );

    $command = 

        'SERVERURL="'.$localhost.$serverpath.'" '.

        'SUSPEND="'.SUSPEND.'" '.
        'SUSPENDOPT="'.SUSPENDOPT.'" '.
        'SUSPENDFILE="'.$suspendfile.'" '.

        'WGET="'.WGET.'" '.
        'WGETOPT="'.WGETOPT.'" '.

        'SLEEPSEC="'.SUSPENDSEC.'" '.

        $script
        ;

    show_debug( $command );

    file_put_contents( get_suspendfile(), time().',0' );

    // スクリプトの実行後、wget で do-suspend.php が呼び出されてスクリプトのPIDを取得する
    exec( $command.' > /dev/null 2>&1 &' );

    return TRUE;
}


// サスペンド中止
// サスペンドスクリプトをkillする
function stop_suspend( $pid )
{
    show_debug( 'stop_suspend pid = '.$pid );

    $command = KILLKILLCMD.' '.$pid;
    show_debug( $command );

    exec( $command );
    sleep( WAITTIME );

    unlink( get_suspendfile() );    
}


?>
